---
title: LispKit System
group: Libraries
order: 6
---

## Source files

**(load _filename_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  
**(load _filename_ _environment_)**  

`load` reads a source file specified by _filename_ and executes it in the given _environment_. If no environment is specified, the current _interaction environment_ is used, which can be accessed via `(interaction-environment)`. Execution of the file consists of reading expressions and definitions from the file and evaluating them sequentially in the environment. `load` returns the result of evaluating the last expression or definition from the file.

It is an error if _filename_ is not a string. If _filename_ is not an absolute file path, LispKit will try to find the file in a predefined set of directories, such as the default libraries search path. If no file suffixes are provided, the system is trying to determine the right suffix. For instance, `(load "Prelude")` will find the prelude file, determine its suffix and load and execute the file.


## File operations

Files and directories are referenced by _paths_. Paths are strings consisting of directory names separated by character '/' optionally followed by a file name (if the path refers to a file). Paths are either _absolute_, if they start with character '/', or they are _relative_ to the _current directory_. LispKit supports ways to read and write files, inspect directories, get metadata about files (e.g. file sizes), etc.

**current-directory** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[parameter object]</span>  

Defines the path referring to the _current directory_. Each LispKit virtual machine has its own current directory.

**(file-path _path base_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Constructs a new file or directory path consisting of a base path _base_ and a relative file path _path_. Example: `(file-path "Photos/img01.jpg" "/Users/objecthub")` ⇒ `"/Users/objecthub/Photos/img01.jpg"`.

**(parent-file-path _path_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

If _path_ refers to a file, then `parent-file-path` returns the directory in which this file is contained. If _path_ refers to a directory, then `parent-file-path` returns the directory in which this directory is contained.

**(file-path-root? _path_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns `#t` if _path_ exists and corresponds to the root of the directory hierarchy. The root is typically equivalent to "/". It is an error if _path_ is not a string.

**(file-exists? _filepath_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

The `file-exists?` procedure returns `#t` if the named file exists at the time the procedure is called, and `#f` otherwise. It is an error if _filename_ is not a string.

**(directory-exists? _dirpath_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

The `directory-exists?` procedure returns `#t` if the named directory exists at the time the procedure is called, and `#f` otherwise. It is an error if _filename_ is not a string.

**(file-or-directory-exists? _path_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

The `file-or-directory-exists?` procedure returns `#t` if the named file or directory exists at the time the procedure is called, and `#f` otherwise. It is an error if _filename_ is not a string.

**(delete-file _filepath_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

The `delete-file` procedure deletes the file specified by _filepath_ if it exists and can be deleted. If the file does not exist or cannot be deleted, an error that satisfies `file-error?` is signaled. It is an error if _filepath_ is not a string.

**(delete-directory _dirpath_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

The `delete-directory` procedure deletes the directory specified by _dirpath_ if it exists and can be deleted. If the directory does not exist or cannot be deleted, an error that satisfies `file-error?` is signaled. It is an error if _dirpath_ is not a string.

**(delete-file-or-directory _path_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

The `delete-file-or-directory` procedure deletes the directory or file specified by _path_ if it exists and can be deleted. If _path_ neither leads to a file nor a directory or the file or directory cannot be deleted, an error that satisfies `file-error?` is signaled. It is an error if _path_ is not a string.

**(copy-file _filepath targetpath_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

The `copy-file` procedure copies the file specified by _filepath_ to the file specified by _targetpath_. An error satisfying `file-error?` is signaled if _filepath_ does not lead to an existing file or if a file at _targetpath_ cannot be written. It is an error if either _filepath_ or _targetpath_ are not strings.

**(copy-directory _dirpath targetpath_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

The `copy-directory` procedure copies the directory specified by _dirpath_ to the directory specified by _targetpath_. An error satisfying `file-error?` is signaled if _dirpath_ does not lead to an existing directory or if a directory at _targetpath_ cannot be written. It is an error if either _dirpath_ or _targetpath_ are not strings.

**(copy-file-or-directory _sourcepath targetpath_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

The `copy-file-or-directory` procedure copies the file or directory specified by _sourcepath_ to the file or directory specified by _targetpath_. An error satisfying `file-error?` is signaled if _sourcepath_ does not lead to an existing file or directory, or if a file or directory at _targetpath_ cannot be written. It is an error if either _sourcepath_ or _targetpath_ are not strings.

**(move-file _filepath targetpath_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Moves the file at _filepath_ to _targetpath_. This procedure fails if _filepath_ does not reference an existing file, or if the file cannot be moved to _targetpath_. It is an error if either _filepath_ or _targetpath_ are not strings.

**(move-directory _dirpath targetpath_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Moves the directory at _dirpath_ to _targetpath_. This procedure fails if _dirpath_ does not reference an existing directory, or if the directory cannot be moved to _targetpath_. It is an error if either _dirpath_ or _targetpath_ are not strings.

**(move-file-or-directory _sourcepath targetpath_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Moves the file or directory at _sourcepath_ to _targetpath_. This procedure fails if _sourcepath_ does not reference an existing file or directory, or if the file or directory cannot be moved to _targetpath_. It is an error if either _sourcepath_ or _targetpath_ are not strings.

**(file-size _filepath_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the size of the file specificed by _filepath_ in bytes. It is an error if _filepath_ is not a string or if _filepath_ does not reference an existing file.

**(directory-list _dirpath_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns a list of names of files and directories contained in the directory specified by _dirpath_. It is an error if _dirpath_ is not a string or if _dirpath_ does not reference an existing directory.

**(make-directory _dirpath_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Creates a directory with path _dirpath_. If the directory exists already or if it is not possible to create a directory with path _dirpath_, `make-directory` fails with an error. It is an error if _dirpath_ is not a string.

**(open-file _filepath_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  
**(open-file _filepath app_)**  
**(open-file _filepath app activate_)**  

Opens the file specified by _filepath_ with the application _app_. _activate_ is a boolean argument. If it is `#t`, it will make _app_ the frontmost application after invoking it. If _app_ is not specified, the default application for the type of the file specified by _filepath_ is used. If _activate_ is not specified, it is assumed it is `#t`. `open-file` returns `#t` if it was possible to open the file, `#f` otherwise. Example: `(open-file "/Users/objecthub/main.swift" "TextEdit")`.


## Network operations

**(open-url _url_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Opens the given url in the default browser and makes the browser the frontmost application.

**(http-get _url_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  
**(http-get _url timeout_)**  

`http-get` performs an `http get` request for the given URL. `timeout` is a floating point number defining the time in seconds it should take at most for receiving a response. `http-get` returns two values: the HTTP header in form of an association list, and the content in form of a bytevector. It is an error if the `http get` request fails. Example:

`(http-get "http://github.com/objecthub")`
⇒ 
{% highlight scheme %}
(("Date" . "Sat, 17 Nov 2018 22:47:19 GMT")
 ("Referrer-Policy" . "origin-when-cross-origin, strict-origin-when-cross-origin")
 ("X-XSS-Protection" . "1; mode=block")
 ("Status" . "200 OK")
 ("Transfer-Encoding" . "Identity")
 ...
 ("Content-Type" . "text/html; charset=utf-8")
 ("Server" . "GitHub.com"))
#u8(10 10 60 33 68 79 67 84 89 80 69 32 104 116 109 108 62 10 60 104 116 109
    108 32 108 97 110 103 61 34 101 110 34 62 10 32 32 60 104 101 97 100 62
    10 32 32 32 32 60 109 101 116 97 32 99 104 97 114 115 101 116 61 34 117
    116 102 ...)
{% endhighlight %}


## Time operations

**(time _expr_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[syntax]</span>  

`time` compiles _expr_ and executes it. The form displays the time it took to execute _expr_ as a side-effect. It returns the result of executing _expr_.

**(current-second)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns a floating-point number representing the current time on the International Atomic Time (TAI) scale. The value `0.0` represents midnight on January 1, 1970 TAI (equivalent to ten seconds before midnight UTC) and the value `1.0` represents one TAI second later.
**Note:** The current implementation returns the same number like `current-seconds`. This is not conforming to the R7RS spec requiring TAI scale.

**(current-jiffy)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the number of jiffies as a fixnum that have elapsed since an arbitrary epoch. A jiffy is a fraction of a second which is defined by the return value of the `jiffies-per-second` procedure. The starting epoch is guaranteed to be constant during a run of the program, but may vary between runs.

**(jiffies-per-second)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns a fixnum representing the number of jiffies per SI second. Here is an example for how to use `jiffies-per-second`:

{% highlight scheme %}
(define (time-length)
  (let ((list (make-list 100000))
        (start (current-jiffy)))
    (length list)
    (/ (- (current-jiffy) start) (jiffies-per-second))))
{% endhighlight %}


## Locales

For handling locale-specific behavior, e.g. for formatting numbers and dates, library `(lispkit system)` defines a framework in which

   - regions/countries are identified via ISO 3166-1 Alpha 2-code strings,
   - languages are identified via ISO 639-1 2-letter strings, and
   - locales (i.e. combinations of regions and languages) are identified as symbols.

Library `(lispkit system)` provides functions for returning all available regions, languages, and locales. It also defines functions to map identifiers to human-readable names and to construct identifiers out of other identifiers.

**(available-regions)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns a list of 2-letter region code identifiers (strings) for all available region.

**(region-name _ident_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  
**(region-name _ident locale_)**  

Returns the name of the region identified by the 2-letter region code string _ident_ for the given locale _locale_. If _locale_ is not provided, the current (system-provided) locale is used.

**(available-languages)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns a list of 2-letter language code identifiers (strings) for all available languages.

**(language-name _ident_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  
**(language-name _ident locale_)** 

Returns the name of the language identified by the 2-letter language code string _ident_ for the given locale _locale_. If _locale_ is not provided, the current (system-configured) locale is used.

**(available-locales)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns a list of all available locale identifiers (symbols).

**(available-locale? _locale_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns `#t` if the symbol _locale_ is identifying a locale supported by the operating system; returns `#f` otherwise.

**(locale)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  
**(locale _lang_)**  
**(locale _lang country_)**  

If no argument is provided `locale` returns the current locale which got configured by the user for the operation system. If argument _lang_ is provided, a locale representing _lang_ (and all countries for which _lang_ is supported) is returned. If both _lang_ and _country_ are provided, `locale` will return a symbol identifying the corresponding locale.

This function never fails if both _lang_ and _country_ are strings. It can be used for constructing locales that are not supported by the underlying operating system. This can be checked with function `available-locale?`.

**(locale-region _locale_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the 2-letter region code string for the region targeted by the locale identifier _locale_. If _locale_ does not target a region, `locale-region` returns `#f`.

**(locale-language _locale_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the 2-letter language code string for the language targeted by the locale identifier _locale_. If _locale_ does not target a language, `locale-language` returns `#f`.

**(locale-currency _locale_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the 3-letter currency code string for the currency associated with the country targeted by _locale_. If _locale_ does not target a country, `locale-currency` returns `#f`.


## Execution environment

**(get-environment-variable _name_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Many operating systems provide each running process with an environment consisting of _environment variables_. Both the name and value of an environment variable are represented as strings. The procedure `get-environment-variable` returns the value of the environment variable _name_, or `#f` if the named environment variable is not found. Example: `(get-environment-variable "PATH")` ⇒ `"/usr/local/bin:/usr/bin:/bin"`.

**(get-environment-variables)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the names and values of all the environment variables as an association list, where the car of each entry is the name of an environment variable and the cdr is its value, both as strings. Example: `(("USER" . "root") ("HOME" . "/"))`.

**(command-line)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the command line passed to the process as a list of strings. The first string corresponds to the command name.

**(features)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns a list of the feature identifiers which `cond-expand` treats as true. Here is an example of what `features` might return: `(modules x86-64 lispkit macosx syntax-rules complex 64bit macos little-endian dynamic-loading ratios r7rs)`. LispKit supports at least the following feature identifiers:

  * lispkit
  * r7rs
  * ratios
  * complex
  * syntax-rules
  * little-endian
  * big-endian
  * dynamic-loading
  * modules
  * 32bit
  * 64bit
  * macos
  * macosx
  * ios
  * linux
  * i386
  * x86-64
  * arm64
  * arm

**(implementation-name)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the name of the Scheme implementation. For LispKit, this function returns the string "LispKit".

**(implementation-version)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the version of the Scheme implementation as a string.

**(cpu-architecture)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the CPU architecture on which this Scheme implementation is executing as a string.

**(machine-name)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns a name for the particular machine on which the Scheme implementation is currently running.

**(machine-model)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns an identifier for the machine on which the Scheme implementation is currently running.

**(os-type)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the type of the operating system on which the Scheme implementation is running as a string. For macOS, this procedure returns "Darwin".

**(os-name)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the name of the operating system on which the Scheme implementation is running as a string. For macOS, this procedure returns "macOS".

**(os-version)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the build number of the operating system on which the Scheme implementation is running as a string. For macOS 10.14.1, this procedure returns "18B75".

**(os-release)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the (major) release version of the operating system on which the Scheme implementation is running as a string. For macOS 10.14.1, this procedure returns "10.14".

**(current-user-name)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns the username of the user running the Scheme implementation as a string.

**(user-data _username_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns information about the user specified via _username_ in form of a list. The list provides the following information in the given order:

   1. User id (fixnum)
   2. Group id (fixnum)
   3. Username (string)
   4. Full name (string)
   5. Home directory (string)
   6. Default shell (string)

Here is an example showing the result for invocation `(user-data "objecthub")`: `(501 20 "objecthub" "Max Mustermann" "/Users/objecthub/" "/bin/bash")`.


## Scheme environment

**(gc)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Force garbage collection to be performed.

**(compile _expr_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Compiles expression _expr_ and displays the disassembled code. This is what is being printed when executing `(compile '(do ((i 0 (fx1+ i)))((fx> i 10))(display i)(newline)))`:

{% highlight scheme %}
CONSTANTS:
    0: #<procedure display>
    1: #<procedure newline>
INSTRUCTIONS:
    0: alloc 1
    1: push_fixnum 0
    2: make_local_variable 0
    3: push_local_value 0
    4: push_fixnum 10
    5: fx_gt
    6: branch_if 14                    ;; jump to 20
    7: make_frame
    8: push_constant 0                 ;; #<procedure display>
    9: push_local_value 0
   10: call 1
   11: pop
   12: make_frame
   13: push_constant 1                 ;; #<procedure newline>
   14: call 0
   15: pop
   16: push_local_value 0
   17: fx_inc
   18: set_local_value 0
   19: branch -16                      ;; jump to 3
   20: push_void
   21: reset 0, 1
   22: return
{% endhighlight %}

**(disassemble _proc_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Disassembles procedure _proc_ and prints out the code. This is what is being printed when executing `(disassemble caddr)`:

{% highlight scheme %}
CONSTANTS:
INSTRUCTIONS:
    0: assert_arg_count 1
    1: push_global 426
    2: make_frame
    3: push_global 431
    4: push_local 0
    5: call 1
    6: tail_call 1
{% endhighlight %}

**(trace-calls)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  
**(trace-calls _level_)**  

This function is used to enable/disable call tracing. When call tracing is enabled, all function calls that are executed by the virtual machine are being printed to the console. Call tracing operates at three levels:

   - `0`: Call tracing is switched off
   - `1`: Call tracing is enabled only for procedures for which it is enabled (via function `set-procedure-trace!`)
   - `2`: Call tracing is switched on for all procedures (independent of procedure-level tracing being enabled or disabled)

`(trace-calls n)` will set call tracing to level `n`. If the level is ommitted, `trace-calls` will return the current call tracing level.

For instance, if call tracing is enabled via `(trace-calls 2)`, executing `(fib 3)` will print the following call trace.

{% highlight scheme %}
> (define (fib n)
    (if (< n 2) n (+ (fib (- n 1)) (fib (- n 2)))))
> (trace-calls 2)
> (fib 2)
  ↪︎ (fib 2) in <repl>
    ⟶ (< 2 2) in fib
    ⟵ #f from <
    ⟶ (- 2 1) in fib
    ⟵ 1 from -
    ⟶ (fib 1) in fib
      ⟶ (< 1 2) in fib
      ⟵ #t from < in fib
    ⟵ 1 from fib in fib
    ⟶ (- 2 2) in fib
    ⟵ 0 from -
    ⟶ (fib 0) in fib
      ⟶ (< 0 2) in fib
      ⟵ #t from < in fib
    ⟵ 0 from fib in fib
  ↪︎ (+ 1 0) in fib
  ⟵ 1 from fib
1
{% endhighlight %}

Function invocations are prefixed with `⟶`,  or `↪` if it's a tail call. The value returned by a function call is prefixed with `⟵`.

**(procedure-trace? _proc_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns `#f` if procedure-level call tracing is disabled for _proc_, `#t` otherwise.

**(set-procedure-trace! _proc trace?_)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Enables procedure-level call tracing for procedure _proc_ if _trace?_ is set to `#t`. It disables call tracing for _proc_ if _trace?_ is `#f`.

**(available-symbols)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns a list of all symbols that have been used so far.

**(loaded-libraries)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns a list of all libraries that have been loaded so far.

{% highlight scheme %}
> (loaded-libraries)
((lispkit draw) (lispkit base) (lispkit port) (lispkit control) (lispkit type) (lispkit list) (lispkit string)
 (lispkit math) (lispkit date-time) (lispkit dynamic) (lispkit char-set) (lispkit bytevector)
 (lispkit char) (lispkit vector) (lispkit regexp) (lispkit record) (lispkit hashtable)
 (lispkit system) (lispkit core) (lispkit gvector) (lispkit box))
{% endhighlight %}

**(loaded-sources)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Returns a list of all sources that have been loaded.

**(environment-info)** &nbsp;&nbsp;&nbsp; <span style="float:right;text-align:rigth;">[procedure]</span>  

Prints out debug information about the current execution environment (mostly relevant for developing LispKit).
